<?php
require_once __DIR__ . '/app.controller.php';
require_once __DIR__ . '/../lib/users.class.php';

class UserController extends AppController
{

    public function __construct()
    {
        $users = new Users();
        if (!$users->validateUser()) {
            self::throwForbiddenRequestHTTPError();
        }
    }

    public static function login()
    {
        $email = $_POST['email'] ?? null;
        $password = $_POST['password'] ?? null;
        $remember = $_POST['rememberMe'] ?? null;
        if ($email !== null && $password !== null) {
            $users = new Users();
            try {
                $response = $users->login($email, $password, $remember !== null ? true : false);
                self::json($response);
            } catch (Exception $e) {
                self::throwHTTPError($e->getMessage(), $e->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public static function sendTFACode()
    {
        $email = $_POST['email'] ?? null;
        $token = $_POST['token'] ?? null;
        if ($email !== null && $token !== null) {
            $users = new Users();
            
            try {
                $response = $users->sendTFACode($email, $token);
                self::json($response);
            } catch (Exception $e) {
                self::throwHTTPError($e->getMessage(), $e->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public static function verifyTFACode()
    {
        $token = $_POST['token'] ?? null;
        $code = $_POST['code'] ?? null;
        $remember = $_POST['rememberMe'] ?? null;
        if ($token !== null && $code !== null) {
            $users = new Users();
            try {
                $response = $users->verifyTFA($code, $token, $remember !== null ? true : false);
                self::json($response);
            } catch (Exception $e) {
                self::throwHTTPError($e->getMessage(), $e->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public static function isLogged()
    {
        $users = new Users();
        self::json(['success' => $users->validateUser()]);
    }

    public static function signUp()
    {

        // user signup
        $firstName = $_POST['firstName'] ?? null;
        $lastName = $_POST['lastName'] ?? null;
        $email = $_POST['email'] ?? null;
        $password = $_POST['password'] ?? null;
        $username = $_POST['username'] ?? null;
        $country = $_POST['country'] ?? null;
        $phone = $_POST['phone'] ?? null;
        $ref = $_POST['ref'] ?? null;
        if ($firstName !== null && $lastName !== null && $email !== null && $username !== null  && $country !== null  && $phone !== null && $password) {
            $users = new Users();
            try {
                $passwordText = $password === null ? '' : $password;
                self::json($users->signUp($firstName, $lastName, $email, $username, $country, $phone, $password, $ref));
            } catch (Exception $e) {
                self::throwHTTPError($e->getMessage(), $e->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public static function verifyEmail()
    {

        $token = $_GET['token'] ?? null;
        if ($token !== null) {
            $users = new Users();
            try {
                self::json($users->verifyEmail($token));
            } catch (Exception $e) {
                self::throwHTTPError($e->getMessage(), $e->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public static function forgotPassword()
    {
        $email = $_GET['email'] ?? null;
        if ($email !== null) {
            $users = new Users();
            try {
                self::json($users->forgotPassword($email));
            } catch (\Throwable $th) {
                self::throwHTTPError($th->getMessage(), $th->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public static function resetPassword()
    {
        $token = $_POST['token'] ?? null;
        $password = $_POST['password'] ?? null;

        if ($token !== null && $password !== null) {
            $users = new Users();
            try {
                self::json($users->resetPassword($token, $password));
            } catch (\Throwable $th) {
                self::throwHTTPError($th->getMessage(), $th->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }



    public function dashboard()
    {
        try {
            $users = new Users();
            self::json($users->dashBoard());
        } catch (Exception $e) {
            self::throwHTTPError($e->getMessage(), $e->getCode());
        }
    }

    public function changePassword()
    {
        $old = $_POST['old'] ?? null;
        $new = $_POST['new'] ?? null;

        if ($old !== null && $new !== null) {
            try {
                $users = new Users();
                self::json($users->changePassword($old, $new));
            } catch (Exception $e) {
                self::throwHTTPError($e->getMessage(), $e->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public function uploadCurrencies()
    {
        $currencies = $_POST['currencies'] ?? null;

        if ($currencies != null) {
            $users = new Users();
            $currencies = json_decode($currencies, true);
            self::json($users->uploadCurrencies($currencies));
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public function createInvestmentData()
    {
        $users = new Users();

        echo json_encode([
            'plans' => App::INVESTMENT_PLANS,
            'userCurrencies' => $users->getUserCurrencies(),
        ]);
    }

    public function createInvestment()
    {
        $planId = $_POST['planId'] ?? null;
        $amount = $_POST['amount'] ?? null;
        $currencyId = $_POST['currencyId'] ?? null;
        $wireTransferDetails = $_POST['wireTransferDetails'] ?? null;
        if ($planId !== null && $amount !== null && ($currencyId !== null || $wireTransferDetails !== null)) {
            if($currencyId !== null){
                $currencyId = (int) $currencyId;
            }
            try {
                $users = new Users();
                self::json($users->createInvestment((int) $planId, (float) $amount, $currencyId, $wireTransferDetails));
            } catch (Exception $e) {
                self::throwHTTPError($e->getMessage(), $e->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public function investments()
    {
        $page = $_GET['page'] ?? null;
        $limit = $_GET['limit'] ?? null;

        if ($page !== null && $limit !== null) {
            $users = new Users();
            self::json($users->getInvestments((int) $page, (int) $limit));
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public function investmentFullDetail()
    {
        $id = $_GET['id'] ?? null;

        if ($id !== null) {
            $users = new Users();
            try {
                self::json($users->getInvestmentFullDetail($id));
            } catch (\Throwable $th) {
                throw new Exception($th->getMessage(), $th->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public function confirmInvestmentPayment()
    {
        $id = $_POST['id'] ?? null;
        if ($id !== null) {
            $users = new Users();
            self::json($users->confirmInvestmentPayment((int) $id));
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public function payInvestmentWithBalance()
    {
        $id = $_POST['id'] ?? null;
        if ($id !== null) {
            $users = new Users();
            self::json($users->payInvestmentWithBalance((int) $id));
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public function claimReferralBonus()
    {
        $id = $_POST['id'] ?? null;
        if ($id !== null) {
            $users = new Users();
            self::json($users->claimReferralBonus((int) $id));
        } else {
            self::throwBadRequestHTTPError();
        }
    }



    public function withdrawals()
    {
        $page = $_GET['page'] ?? null;
        $limit = $_GET['limit'] ?? null;

        if ($page !== null && $limit !== null) {
            $users = new Users();
            self::json($users->getWithdrawals((int) $page, (int) $limit));
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public function unclaimReferralBonus()
    {
        $users = new Users();
        self::json($users->getAllUnclaimedReferralBonus());
    }

    public function accountTopups()
    {
        $users = new Users();
        self::json($users->getAllAccountTopups());
    }


    public function viewReferralInfo()
    {
        $users = new Users();
        try {
            self::json($users->viewReferralInfo());
        } catch (\Throwable $th) {
            throw new Exception($th->getMessage(), $th->getCode());
        }
    }

    public function requestWithdrawal()
    {

        $amount = $_POST['amount'] ?? null;
        $currencyId = $_POST['currencyId'] ?? null;

        if ($amount !== null) {
            if($currencyId !== null){
                $currencyId = (int) $currencyId;
            }
            $users = new Users();
            try {
                self::json($users->requestWithdrawal((float) $amount, $currencyId));
            } catch (\Throwable $th) {
                throw new Exception($th->getMessage(), $th->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }
    
     public function requestCard()
    {

        $type = $_POST['type'] ?? null;
        $name = $_POST['name'] ?? null;
        $country = $_POST['country'] ?? null;
        $address = $_POST['address'] ?? null;
        $phone = $_POST['phone'] ?? null;
        if ($type !== null && $name !== null && $address !== null && $country !== null && $phone !== null) {
            $users = new Users();
            try {
                self::json($users->requestCard($name, $phone, $country, $address, $type));
            } catch (\Throwable $th) {
                throw new Exception($th->getMessage(), $th->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }

    public function sendVerificationMail()
    {
        $users = new Users();
        try {
            self::json($users->sendVerificationMail());
        } catch (\Throwable $th) {
            throw new Exception($th->getMessage(), $th->getCode());
        }
    }

    public function updateProfilePhoto()
    {
        $file = $_FILES['image'] ?? null;
        if ($file !== null) {
            $users = new Users();
            try {
                self::json($users->updateProfilePhoto($file));
            } catch (\Throwable $th) {
                throw new Exception($th->getMessage(), $th->getCode());
            }
        } else {
            self::throwBadRequestHTTPError();
        }
    }
}
