<?php
class File
{
    private static function cropImage(string $filename, string $directory, string $extension)
    {
      
      $im = null;
        $imageDirectory = $directory . '/' . $filename;
        switch ($extension) {
            case 'png':
                $im = imagecreatefrompng($imageDirectory);
                break;
            case 'jpg':
                $im = imagecreatefromjpeg($imageDirectory);
                break;
            case 'jpeg':
                $im = imagecreatefromjpeg($imageDirectory);
                break;
            case 'gif':
                $im = imagecreatefromgif($imageDirectory);
                break;
            case 'bmp':
                $im = imagecreatefrombmp($imageDirectory);
                break;
            case 'webp':
                $im = imagecreatefromwebp($imageDirectory);
                break;
            case 'gd':
                $im = imagecreatefromgd($imageDirectory);
                break;
        }
        $size = min(imagesx($im), imagesy($im));
        $im2 = imagecrop($im, ['x' => 0, 'y' => 0, 'width' => $size, 'height' => $size]);
        $cropDirectory = $directory . '/crop/' . $filename;
        if ($im2 !== false) {
            if ($extension === 'png') {
                imagepng($im2, $cropDirectory);
            } else if ($extension === 'jpg' || $extension === 'jpeg') {
                imagejpeg($im2, $cropDirectory);
            } else if ($extension === 'gif') {
                imagegif($im2, $cropDirectory);
            } else if ($extension === 'bmp') {
                imagebmp($im2, $cropDirectory);
            } else if ($extension === 'webp') {
                imagewebp($im2, $cropDirectory);
            } else if ($extension === 'gd') {
                imagegd($im2, $cropDirectory);
            }

            imagedestroy($im2);
        }
        imagedestroy($im);
    }
    public static function getRootDirectory(): string{
        return dirname(dirname(__FILE__));
    }

    public static function uploadFile(array $file, string $directory, array $accept = [], bool $rename = true, bool $crop = false): array
    {
        $result = [];

        

        if ($file['error'] === 0) {
            $pathInfo = pathinfo($file['name']);
            if (count($accept) > 0) {
                if (!in_array($pathInfo['extension'], $accept)) {

                    throw new Exception('Invalid File Extension only ' . implode(',', $accept) . ' is allowed');
                }
            }

           
            if (is_dir(File::getRootDirectory() . "/" . $directory)) {
                $fileName = $file['name'];
                if ($rename) {
                    do {
                        $fileName = uniqid("es-",true) . "." . $pathInfo['extension'];
                    } while (file_exists(File::getRootDirectory() . "/" . $directory . "/" . $fileName));
                }
                if (move_uploaded_file($file['tmp_name'], File::getRootDirectory() . "/" . $directory . "/" . $fileName)) {
                    $result = [
                        'filename' =>$fileName,
                        'directory' => $directory,
                    ];
                    File::cropImage($fileName, File::getRootDirectory() . "/" . $directory, $pathInfo['extension']);
                }
            } else {
                throw new Exception("Directory '".File::getRootDirectory()."/$directory' does not exists");
            }
        }
        return $result;
    }

    public static function deleteFile(string $file, string $path): void{
        if(file_exists(File::getRootDirectory() . "/" . $path . "/" . $file)) {
            unlink(File::getRootDirectory() . "/" . $path . "/" . $file);
        }
    }
}
