<?php
class Paginator
{
    private $conn;
    private $itemsPerPage;

    public function __construct($conn, int $itemsPerPage)
    {
        $this->conn = $conn;
        $this->itemsPerPage = $itemsPerPage;
    }
    public function paginate(int $page, string $sql): array
    {
        $response = [];
        $sql = str_replace(';', '', $sql);

        try {
            $prepStmt = $this->conn->prepare($sql);
            
            $prepStmt->execute();
            $result = $prepStmt->get_result();
            $totalRecords = $result->num_rows;

            $pages = ceil($totalRecords / $this->itemsPerPage);
            $offset = ($page - 1) * $this->itemsPerPage;

            $sql .= " LIMIT $offset," . $this->itemsPerPage;

            $prepStmt = $this->conn->prepare($sql);
            if ($prepStmt->execute()) {
                $result = $prepStmt->get_result();
                $response['data'] = $result->fetch_all(MYSQLI_ASSOC);
                $response['info'] = [
                    'page' => $page,
                    'pages' => $pages,
                    'itemsPerPage' => $this->itemsPerPage,
                    'totalRecords' => $totalRecords,
                ];
            }

        } catch (Exception $e) {
            throw new Exception($e);
        }
        return $response;
    }
}
