<?php
require_once 'app.class.php';
require_once 'database.class.php';
require_once 'file.class.php';
require_once 'errors.class.php';
require_once 'users.class.php';
require_once 'paginator.class.php';
require_once 'mailer.class.php';
require_once 'transactions.class.php';

class Admin extends App
{
    private $conn;
    private const ADMIN_TOKEN_SECRET = 'cryptoInv&estmentAdmin#@s';
    private const APP_SECRET = "%^4shaysyd";
    private const AUTH_HEADER_NAME = 'Authorization';
    public $adminInfo = null;

    public function __construct()
    {
        $this->conn = Database::connect();
        $this->validateAdmin();
    }

    public function validateAdmin(): bool
    {
        $valid = false;
        $headers = apache_request_headers();
        $token = $headers[Admin::AUTH_HEADER_NAME] ?? null;
        if ($token !== null) {
            $id = App::verifyToken($token, Admin::ADMIN_TOKEN_SECRET);
            if ($id !== null) {
                try {
                    $conn = Database::connect();
                    $sql = "
                    SELECT id, first_name, last_name, email
                    FROM admin WHERE id= ?;
                    ";
                    $prepStmt = $conn->prepare($sql);
                    $prepStmt->bind_param('i', $id);
                    $prepStmt->execute();
                    $result = $prepStmt->get_result();
                    if ($result->num_rows === 1) {
                        $this->adminInfo = $result->fetch_assoc();
                        $valid = true;
                    }
                } catch (Exception $e) {
                    throw new Exception($e);
                }
            }
        }
        return $valid;
    }

    public function dashboard(): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $users = new Users();
            $transactions = new Transactions();

            $response['admin'] = $this->adminInfo;
            $response['users_statistics'] = $users->usersStstistics();
            $response['transaction_statistics'] = $transactions->transactionStatistics();
        }
        return $response;
    }

    public function viewReferralInfo(int $userId)
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $users = new Users();
            $response = $users->viewReferralInfo($userId);
        }
        return $response;
    }

    public function forgotPassword(string $email): array
    {
        $response = ['success' => false];

        $email = trim(strtolower($email));
        $sql = "SELECT first_name, email FROM admin WHERE LOWER(email) = ?";
        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->bind_param('s', $email);
        $prepStmt->execute();
        $result = $prepStmt->get_result();

        if ($result->num_rows === 1) {
            $row = $result->fetch_assoc();

            $token = rawurlencode(App::generateToken($row['email'], "mailSeksjet@sdhfdh", (60 * 60 * 30)));
            $mailer = new Mailer();
            if ($mailer->sendPasswordResetMailAdmin($row['first_name'], $token, $row['email'])) {
                $response['success'] = true;
            } else {
                throw new Exception('Problem sending reset mail',422);
            }
        } else {
            throw new Exception(Errors::adminError('unknownAdmin'),422);
        }
        return $response;
    }

    public function resetPassword(string $token, $password): array
    {
        $response = ['token' => null];

        $payload = App::verifyToken($token, "mailSeksjet@sdhfdh"); //payload is email
        if ($payload !== null) {
            $sql = "SELECT email FROM admin WHERE LOWER(email) = ?";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('s', $payload);
            $prepStmt->execute();
            $result = $prepStmt->get_result();
            if ($result->num_rows === 1) {
                $hash = password_hash($password, PASSWORD_DEFAULT);
                $sql = "UPDATE admin SET password = ? WHERE LOWER(email) = ?";
                $prepStmt = $this->conn->prepare($sql);
                $prepStmt->bind_param('ss', $hash, $payload);
                if ($prepStmt->execute()) {
                    $response = $this->login($payload, $password, true);
                } else {
                    throw new Exception('Problem resetting password',422);
                }
            } else {
                throw new Exception(Errors::adminError('unknownAdmin'),422);
            }
        } else {
            throw new Exception('Invalid Link',422);
        }
        return $response;
    }

    public function getAllAdmin(): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $sql = "SELECT id, first_name, last_name
            FROM admin;
            ";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->execute();
            $result = $prepStmt->get_result();
            $response['data'] = $result->fetch_all(MYSQLI_ASSOC);
        }
        return $response;
    }

    public function login(string $email, string $password, bool $rememberMe): array
    {
        $response = [];
        $sql = "SELECT id, password FROM admin WHERE email = ?;";
        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->bind_param('s', $email);
        $prepStmt->execute();
        $result = $prepStmt->get_result();
        if ($result->num_rows === 1) {
            $row = $result->fetch_assoc();
            $passwordCorrect = password_verify($password, $row['password']);
            if ($passwordCorrect) {
                $expires = $rememberMe ? (60 * 60 * 24 * 31) : (60 * 60 * 24); // a day or a month;
                $token = App::generateToken($row['id'], Admin::ADMIN_TOKEN_SECRET, $expires);
                $response['token'] = $token;
            } else {
                throw new Exception(Errors::appError('wrongCredentials'),401);
            }
        } else {
            throw new Exception(Errors::appError('wrongCredentials'),401);
        }
        return $response;
    }

    public function addAdmin(string $firstName, string $lastName, string $email, string $password, string $appSecret): array
    {
        $response = [];
        if (Admin::APP_SECRET === $appSecret) {
            $sql = "SELECT id FROM admin WHERE email = ?;";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('s', $email);
            $prepStmt->execute();
            $result = $prepStmt->get_result();
            if ($result->num_rows === 0) {
                $passwordHash = password_hash($password, PASSWORD_DEFAULT);
                $sql = "INSERT INTO admin
                (first_name, last_name, email, password)
                VALUES(?, ?, ?, ?);";
                $prepStmt = $this->conn->prepare($sql);
                $prepStmt->bind_param('ssss', $firstName, $lastName, $email, $passwordHash);
                if ($prepStmt->execute()) {
                    $response = $this->login($email, $password, false);
                } else {
                    throw new Exception(Errors::adminError('addAdminError'),422);
                }
            } else {
                throw new Exception(Errors::adminError('duplicateEmail'),422);
            }
        } else {
            throw new Exception(Errors::adminError('wrongAppSecret'),422);
        }
        return $response;
    }

    public function changePassword(string $old, string $new): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $adminId = $this->adminInfo['id'];
            $sql = "SELECT `password` FROM admin WHERE id = ?";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('i', $adminId);
            $prepStmt->execute();
            $result = $prepStmt->get_result();
            $row = $result->fetch_assoc();
            if (password_verify($old, $row['password'])) {
                $hash = password_hash($new, PASSWORD_DEFAULT);
                $sql = "UPDATE admin SET `password` = ? WHERE id = ?";
                $prepStmt = $this->conn->prepare($sql);
                $prepStmt->bind_param('si', $hash, $adminId);
                $response['success'] = $prepStmt->execute();
            } else {
                throw new Exception(Errors::appError('incorrectPassword'),401);
            }
        }
        return $response;
    }
    public function getAllUsers(int $page, int $itemsPerPage, string $email): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $users = new Users();
            $response = $users->getAllUsers($page, $itemsPerPage, $email);
        }
        return $response;
    }

    public function getInvestments(int $page, int $limit, string $email = ''): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $transactions = new Transactions();
            $response = $transactions->getInvestments($page, $limit, null, $email);
        }
        return $response;
    }

    public function getInvestmentFullDetail(int $id): array
    {
        $transactions = new Transactions();
        $response = $transactions->investmentFullDetail($id);
        return $response;
    }

    public function approveInvestmentPayment(int $id): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $transactions = new Transactions();
            $response = $transactions->approveInvestmentPayment($id);
        }
        return $response;
    }

    public function deleteInvestmentPayment(int $id): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $transactions = new Transactions();
            $response = $transactions->deleteInvestmentPayment($id);
        }
        return $response;
    }

    public function addProfitToInvestment(int $id, float $amount): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $transactions = new Transactions();
            $response = $transactions->addProfitToInvestment($id, $amount);
            if ($response['error']) {
                throw  new Exception($response['error'], 422);
            } else {
                return ['success' => $response['success']];
            }
        }
        return $response;
    }

    public function approveWithdrawalRequest(int $id): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $transactions = new Transactions();
            $response = $transactions->approveWithdrawalRequest($id);
        }
        return $response;
    }

    public function deleteWithdrawalRequest(int $id): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $transactions = new Transactions();
            $response = $transactions->deleteWithdrawalRequest($id);
        }
        return $response;
    }

    public function toggleSuspendUser(int $userId): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $users = new Users();
            $response = $users->toggleSuspend($userId);
        }
        return $response;
    }


    public function deleteUser(int $userId): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $users = new Users();
            $response = $users->deleteUser($userId);
        }
        return $response;
    }

    public function getWithdrawals(int $page, int $limit, $email = ''): array
    {
        $response = [];
        if ($this->adminInfo !== null) {
            $transactions = new Transactions();
            $response = $transactions->getWithdrawals($page, $limit, null, $email);
        }
        return $response;
    }
}
