<?php
require_once 'app.class.php';
require_once 'database.class.php';
require_once 'errors.class.php';
require_once 'file.class.php';
require_once 'mailer.class.php';
require_once 'transactions.class.php';

class Users extends App
{
    private $conn;
    private const USER_TOKEN_SECRET = '#sdffddjjfjhd%d';
    private const AUTH_HEADER_NAME = 'Authorization';
    public $userInfo = null;

    private $year;
    private $month;
    private $daysInMonth;
    private const MONTHS = [
        'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August',
        'September', 'October', 'November', 'December',
    ];

    public function __construct()
    {
        $this->conn = Database::connect();
        $this->validateUser();
        $this->year = (int) date('Y');
        $this->month = (int) date('m');
        $this->daysInMonth = cal_days_in_month(1, $this->month, $this->year);
    }

    public function validateUser(): bool
    {
        $valid = false;

        $headers = apache_request_headers();
        $token = $headers[Users::AUTH_HEADER_NAME] ?? null;
        if ($token !== null) {
            $id = App::verifyToken($token, Users::USER_TOKEN_SECRET);
            if ($id !== null) {
                try {
                    $conn = Database::connect();
                    $sql = "
                    SELECT id, first_name, last_name, email, country, phone, is_email_verified, `date`, is_suspended, image, currencies_accounts, username
                    FROM users WHERE id = ?;
                    ";
                    $prepStmt = $conn->prepare($sql);
                    $prepStmt->bind_param('i', $id);
                    $prepStmt->execute();
                    $result = $prepStmt->get_result();
                    if ($result->num_rows === 1) {
                        $this->userInfo = $result->fetch_assoc();
                        $valid = true;
                    }
                } catch (Exception $e) {
                    echo $e->getMessage();
                    throw new Exception($e);
                }
            }
        }
        return $valid;
    }
    private function getSignupStatistics(): array
    {
        $response = ['year' => [], 'month' => []];

        //get number of users that sign up each month of the year
        $sql = "SELECT COUNT(id) as total_sign_up,
        MONTH(`date`) as `month`, MONTHNAME(`date`) as `month_name`
        FROM users
        WHERE YEAR(`date`) = " . $this->year . "
        GROUP BY `month`
        ORDER BY `month` ASC;";

        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->execute();
        $result = $prepStmt->get_result();
        $dbData = $result->fetch_all(MYSQLI_ASSOC);
        $data = [];

        for ($c = 1; $c <= count(Users::MONTHS); $c++) {
            $totalSignUp = 0;
            foreach ($dbData as $d) {
                if ($d['month'] === $c) {
                    $totalSignUp = $d['total_sign_up'];
                }
            }
            array_push($data, [
                'month' => USERS::MONTHS[$c - 1],
                'totalSignUp' => $totalSignUp,
            ]);
        }

        $response['year'] = [
            'year' => $this->year,
            'data' => $data,
        ];

        //get number of users that sign up each day of the month

        $sql = "SELECT
        COUNT(id) as total_sign_up, DAY(`date`) as `day`, DAYNAME(`DATE`) as `dayName`
        FROM users
        WHERE YEAR(`date`) = " . $this->year . " AND  MONTH(`date`) =" . $this->month . "
        GROUP BY `day`
        ORDER BY `day` ASC;
        ";

        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->execute();
        $result = $prepStmt->get_result();
        $dbData = $result->fetch_all(MYSQLI_ASSOC);
        $data = [];

        for ($c = 1; $c <= $this->daysInMonth; $c++) {
            $totalSignUp = 0;
            foreach ($dbData as $d) {
                if ($d['day'] === $c) {
                    $totalSignUp = $d['total_sign_up'];
                }
            }
            array_push($data, [
                'day' => $c,
                'totalSignUp' => $totalSignUp,
            ]);
        }

        $response['month'] = [
            'year' => $this->year,
            'month' => $this->month,
            'data' => $data,
        ];
        return $response;
    }

    private function getVisitStatistics(): array
    {
        $response = [];
        //get number of users that visits each month of the year
        $sql = "SELECT MONTH(`date`) as `month`, MONTHNAME(`date`) as `month_name`, ip_addresses, unique_ip_addresses
        FROM visits
        WHERE YEAR(`date`) = " . $this->year . "
        GROUP BY `month`
        ORDER BY `month` ASC;";

        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->execute();
        $result = $prepStmt->get_result();

        $dbData = $result->fetch_all(MYSQLI_ASSOC);
        $data = [];

        for ($c = 1; $c <= count(Users::MONTHS); $c++) {
            $visits = 0;
            $uniqueVisits = 0;
            foreach ($dbData as $d) {
                if ($d['month'] === $c) {
                    $visitsArr = json_decode($d['ip_addresses'], true);
                    $uniqueVisitsArr = json_decode($d['unique_ip_addresses'], true);

                    $visits = count($visitsArr);
                    $uniqueVisitsArr = count($uniqueVisitsArr);
                }
            }
            array_push($data, [
                'month' => USERS::MONTHS[$c - 1],
                'visits' => $visits,
                'uniqueVisits' => $uniqueVisits,
            ]);
        }

        $response['year'] = [
            'year' => $this->year,
            'data' => $data,
        ];

        //get number of users that visits each day of the month
        $sql = "SELECT
        DAY(`date`) as `day`, DAYNAME(`DATE`) as `dayName`, ip_addresses, unique_ip_addresses
        FROM visits
        WHERE YEAR(`date`) = " . $this->year . " AND  MONTH(`date`) =" . $this->month . "
        GROUP BY `day`
        ORDER BY `day` ASC;
        ";

        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->execute();
        $result = $prepStmt->get_result();
        $dbData = $result->fetch_all(MYSQLI_ASSOC);
        $data = [];

        for ($c = 1; $c <= $this->daysInMonth; $c++) {
            $visits = 0;
            $uniqueVisits = 0;
            foreach ($dbData as $d) {
                if ($d['day'] === $c) {
                    $visitsArr = json_decode($d['ip_addresses'], true);
                    $uniqueVisitsArr = json_decode($d['unique_ip_addresses'], true);
                    $visits = count($visitsArr);
                    $uniqueVisits = count($uniqueVisitsArr);
                }
            }
            array_push($data, [
                'day' => $c,
                'visits' => $visits,
                'uniqueVisits' => $uniqueVisits,
            ]);
        }

        $response['month'] = [
            'year' => $this->year,
            'month' => $this->month,
            'data' => $data,
        ];
        return $response;
    }

    public function sendVerificationMail(): array
    {
        $response = ['success' => false];
        $token = rawurlencode(App::generateToken($this->userInfo['email'], "maijsjjset@sdhfdh", (60 * 60 * 30)));
        $mailer = new Mailer();
        $mailer->sendVerifyMail($this->userInfo['first_name'], $token, $this->userInfo['email']);
        $response['success'] = true;
        return $response;
    }

    public function verifyEmail($token): array
    {
        $response = [];
        $token = rawurldecode($token);
        $payload = APP::verifyToken($token, "maijsjjset@sdhfdh");
        if ($payload !== null) {
            $email = $payload;
            $sql = "UPDATE users SET is_email_verified = 1 WHERE email = ?";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('s', $email);
            $prepStmt->execute();
            $response = $this->login($email, '', true, true);
        } else {
           throw new Exception('Invalid Link', 422);
        }
        return $response;
    }

    public function getUser(int $id): array
    {
        $result = [];
        $sql = "
        SELECT id, first_name, last_name, email, is_email_verified, `date`, is_suspended, payment_expires, image
        FROM users WHERE id= ?;
        ";
        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->bind_param('i', $id);
        $prepStmt->execute();
        $result = $prepStmt->get_result();
        if ($result->num_rows === 1) {
            $result = $result->fetch_assoc();
        }
        return $result;
    }

    public function getUserId(?string $emailOrUsername, $isUsername = false): ?int
    {
        $response = null;
        if ($emailOrUsername !== null) {

            $sql = " SELECT id FROM users WHERE ";
            $sql .= $isUsername ? "username" : "email";
            $sql .= " = ?;";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('s', $emailOrUsername);
            $prepStmt->execute();
            $result = $prepStmt->get_result();
            if ($result->num_rows === 1) {
                $row = $result->fetch_assoc();
                $response = $row['id'];
            }
        }

        return $response;
    }

    public function viewReferralInfo(?int $userId = null): array
    {
        $response = [
            'refferredBy' => null,
            'refferrals' => [],
        ];

        $userId = $userId === null ? $this->userInfo['id'] : $userId;

        if ($userId !== null) {
            $sql = "SELECT referral_id FROM users WHERE  id = ?";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('i', $userId);
            $prepStmt->execute();
            $result = $prepStmt->get_result();
            if ($result->num_rows > 0) {
                $row = $result->fetch_assoc();
                $referredById = $row['referral_id'];
                if ($referredById !== null) {
                    $sql = "SELECT id , email, first_name, last_name, `image` FROM users WHERE  id = ?;";
                    $prepStmt = $this->conn->prepare($sql);
                    $prepStmt->bind_param('i', $referredById);
                    $prepStmt->execute();
                    $result = $prepStmt->get_result();
                    $row = $result->fetch_assoc();
                    $response['referredBy'] = $row;
                }
            }

            $sql = "SELECT id , email, first_name, last_name, `image` FROM users WHERE referral_id = ?;";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('i', $userId);
            $prepStmt->execute();
            $result = $prepStmt->get_result();
            if ($result->num_rows > 0) {
                $rows = $result->fetch_all(MYSQLI_ASSOC);
                $response['data']['refferrals'] = $rows;
            }
        } else {
            throw new Exception(Errors::userError('unknownUser'),422);
        }

        return $response;
    }

    public function login(string $emailOrUsername, string $password, bool $rememberMe): array
    {
        $response = [];

        $emailOrUsername = trim($emailOrUsername);

        $sql = "SELECT id, password, is_suspended FROM users WHERE LOWER(email) = ? OR username= ?;";
        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->bind_param('ss', $emailOrUsername, $emailOrUsername);
        $prepStmt->execute();
        $result = $prepStmt->get_result();
        if ($result->num_rows === 1) {
            $row = $result->fetch_assoc();
            if ($row['is_suspended'] == 0) {
                $passwordCorrect = $password === $row['password'];//password_verify($password, $row['password']);
                $expires = $rememberMe ? (60 * 60 * 24 * 31) : (60 * 60 * 24); // a day or a month;
                $token = App::generateToken($row['id'], Users::USER_TOKEN_SECRET, $expires);
                $response['token'] = $token;
            } else {
                throw new Exception(Errors::appError('accountSuspended'), 401);
            }
        } else {
            throw new Exception(Errors::appError('wrongCredentials'), 401);
        }
        return $response;
    }

    public function signUp(string $firstName, string $lastName, string $email, string $username, string $country, string $phone, string $password, $ref = null): array
    {
        $response = [];
        $sql = "SELECT id FROM users WHERE username = ?;";
        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->bind_param('s', $username);
        $prepStmt->execute();
        $result = $prepStmt->get_result();
        $refId = $this->getUserId($ref, true);


        if ($result->num_rows === 0) {
            $sql = "SELECT id FROM users WHERE  email = ?;";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('s', $email);
            $prepStmt->execute();
            $result = $prepStmt->get_result();

            if ($result->num_rows === 0) {
                $passwordHash = $password; // password_hash($password, PASSWORD_DEFAULT);

                $sql = " INSERT INTO users
                (first_name, last_name, email, `password`, is_email_verified, `date`, is_suspended, username, referral_id, country, phone)
                VALUES(?, ?, ?, ?, 0, now(), 0,  ?, ?, ?, ?);
                 ";
                $prepStmt = $this->conn->prepare($sql);
                $prepStmt->bind_param('sssssiss', $firstName, $lastName, $email, $passwordHash, $username, $refId, $country, $phone);
                if ($prepStmt->execute()) {
                    $mailer = new Mailer();
                    $mailer->sendWelcomeMail($firstName, $email);

                    if ($refId != null) {
                        $sql = " SELECT first_name, email FROM users WHERE id = ?";
                        $prepStmt = $this->conn->prepare($sql);
                        $prepStmt->bind_param('i', $refId);
                        $prepStmt->execute();
                        $result = $prepStmt->get_result();
                        if ($result->num_rows > 0) {
                            $row = $result->fetch_assoc();
                            $mailer->sendReferralMail($row['email'], $row['first_name'], $firstName);
                        }
                    }


                    $response = $this->login($email, $password, true);
                } else {
                    throw new Exception(Errors::userError('addUserError'), 422);
                }
            } else {
                throw new Exception(Errors::userError('duplicateEmail'), 422);
            }
        } else {
            throw new Exception(Errors::userError('duplicateUsername'), 422);
        }
        return $response;
    }

    public function getUserCurrencies(): array
    {
        $response = [];
        $id = $this->userInfo['id'];
        $sql = " SELECT currencies_accounts FROM users WHERE id = ? ";
        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->bind_param('i', $id);
        $prepStmt->execute();
        $result = $prepStmt->get_result();
        if ($result->num_rows === 1) {
            $row = $result->fetch_assoc();
            $currencies = [];
            if ($row['currencies_accounts'] !== null) {
                $currencies = json_decode($row['currencies_accounts'], true);
            }
            foreach ($currencies as $c) {
                $currencyIds = array_map(function ($currency) {
                    return $currency['id'];
                }, App::CURRENCIES);

                $i = array_search($c['currencyId'], $currencyIds);
                if ($i >= 0) {
                    array_push($response, [
                        'currency' => App::CURRENCIES[$i],
                        'walletAddress' => $c['walletAddress'],
                    ]);
                }
            }
        }
        return $response;
    }

    public function dashBoard(): array
    {
        $response = [];
        if ($this->userInfo !== null) {
            $response['user'] = $this->userInfo;
            $response['currencies'] = $this->getUserCurrencies();
            $transactions = new Transactions();
            $response['refLink'] = "https://www." . App::SETTINGS['domain'] . "/signup/" . $this->userInfo['username'];
            $response['transactions'] = $transactions->getAccountBalanceSummary($this->userInfo['id']);
            $response['chart'] = $transactions->investmentChartForYear($this->userInfo['id']);
            $response['completions'] = [
                'isEmailVerified' => $this->userInfo['is_email_verified'] === 1,
                'isPhotoUploaded' => $this->userInfo['image'] !== null,
                'isCurrencyUploaded' => $this->userInfo['currencies_accounts'] !== null,
            ];
            $transactions->addProfitToInvestmentAutomatically($this->userInfo['id'], $this->userInfo['email']);
        }
        return $response;
    }

    public function getAllUsers(int $page, int $itemsPerPage, $email = ''): array
    {
        $sql = "SELECT id, first_name, last_name, email, country, phone, `date`, image, is_suspended
        FROM users
        ";
        $sql .= trim($email) !== '' ? " WHERE email LIKE '%" . $this->conn->real_escape_string($email) . "%'" : "";
        $paginator = new Paginator($this->conn, $itemsPerPage);
        return $paginator->paginate($page, $sql);
    }

    public function forgotPassword(string $email): array
    {
        $response = ['success' => false];

        $email = trim(strtolower($email));
        $sql = "SELECT first_name, email FROM users WHERE LOWER(email) = ?";
        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->bind_param('s', $email);
        $prepStmt->execute();
        $result = $prepStmt->get_result();

        if ($result->num_rows === 1) {
            $row = $result->fetch_assoc();

            $token = rawurlencode(App::generateToken($row['email'], "mailSecret@sdhfdh", (60 * 60 * 30)));
            $mailer = new Mailer();
            if ($mailer->sendPasswordResetMail($row['first_name'], $token, $row['email'])) {
                $response['success'] = true;
            } else {
                throw new Exception('Problem Sending Reset Mail', 422);
            }
        } else {
            throw new Exception(Errors::userError('unknownUser'), 422);
        }
        return $response;
    }

    public function resetPassword(string $token, $password): array
    {
        $response = ['token' => null];

        $payload = App::verifyToken($token, "mailSecret@sdhfdh"); //payload is email
        if ($payload !== null) {
            $sql = "SELECT email FROM users WHERE LOWER(email) = ?";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('s', $payload);
            $prepStmt->execute();
            $result = $prepStmt->get_result();
            if ($result->num_rows === 1) {
                $hash = password_hash($password, PASSWORD_DEFAULT);
                $sql = "UPDATE users SET password = ? WHERE LOWER(email) = ?";
                $prepStmt = $this->conn->prepare($sql);
                $prepStmt->bind_param('ss', $hash, $payload);
                if ($prepStmt->execute()) {
                    $response = $this->login($payload, $password, true);
                } else {
                    throw new Exception('Problem Resetting Password', 422);
                }
            } else {
                throw new Exception(Errors::userError('unknownUser'), 422);
            }
        } else {
            throw new Exception('Invalid Link', 422);
        }
        return $response;
    }

    public function changePassword(string $old, string $new): array
    {
        if ($this->userInfo !== null) {
            $userId = $this->userInfo['id'];
            $sql = "SELECT `password` FROM users WHERE id = ?";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('i', $userId);
            $prepStmt->execute();
            $result = $prepStmt->get_result();
            $row = $result->fetch_assoc();
            if (password_verify($old, $row['password'])) {
                $hash = password_hash($new, PASSWORD_DEFAULT);
                $sql = "UPDATE users SET `password` = ? WHERE id = ?";
                $prepStmt = $this->conn->prepare($sql);
                $prepStmt->bind_param('si', $hash, $userId);
                $response['success'] = $prepStmt->execute();
            } else {
                throw new Exception(Errors::appError('incorrectPassword'), 422);
            }
        }
        return $response;
    }

    public function updateProfilePhoto(array $image): array
    {
        $response = [];
        if ($this->userInfo !== null) {
            $id = $this->userInfo['id'];
            $sql = "SELECT id, image FROM users WHERE id=?;";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('i', $id);
            $prepStmt->execute();
            $result = $prepStmt->get_result();
            if ($result->num_rows === 1) {
                $row = $result->fetch_assoc();
                $file = File::uploadFile($image, 'uploads/images/users', ['jpg', 'png', 'jpeg'], true, true);
                $image = $file['filename'];
                $sql = "UPDATE users SET
                    `image` = ? WHERE id = ?;
                ";
                $prepStmt = $this->conn->prepare($sql);
                $prepStmt->bind_param('si', $image, $id);
                $prepStmt->execute();
                if ($prepStmt->execute()) {
                    if ($row['image'] !== null) {
                        File::deleteFile($row['image'], App::UPLOADS_DIR_PATH['images']['users']);
                    }
                    $response['data'] = $image = $file['filename'];
                }
            } else {
                throw new Exception(Errors::userError('unknownUser'), 422);
            }
        }

        return $response;
    }

    public function uploadCurrencies(array $currencies): array
    {
        $response = ['success' => false];
        if ($this->userInfo !== null) {
            $currencyIds = array_map(function ($currency) {
                return $currency['id'];
            }, App::CURRENCIES);
            foreach ($currencies as $currency) {
                if (!in_array($currency['currencyId'], $currencyIds)) {
                    $response['error'] = Errors::transactionError('invalidCurrency');
                    break;
                }
            }

            if ($response['error'] === null) {

                $d = json_encode($currencies);
                $id = $this->userInfo['id'];
                $sql = "UPDATE users SET currencies_accounts = ? WHERE id=?;";
                $prepStmt = $this->conn->prepare($sql);
                $prepStmt->bind_param('si', $d, $id);
                $response['success'] = $prepStmt->execute();
            }
        }

        return $response;
    }

    public function toggleSuspend(int $userId): array
    {
        $response = [];

        $sql = "SELECT is_suspended FROM users WHERE id = ?";
        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->bind_param('i', $userId);
        $prepStmt->execute();
        $result = $prepStmt->get_result();
        if ($result->num_rows > 0) {
            $row = $result->fetch_assoc();
            $suspend = $row['is_suspended'] === 1 ? 0 : 1;
            $sql = "UPDATE users SET is_suspended = ? WHERE id = ?";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('ii', $suspend, $userId);
            if ($prepStmt->execute()) {
                $response['success'] = true;
            }
        } else {
            throw new Exception(Errors::userError('unknownUser'), 422);
        }

        return $response;
    }

    public function deleteUser(int $userId): array
    {
        $response = [];

        $sql = "SELECT id FROM users WHERE id = ?";
        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->bind_param('i', $userId);
        $prepStmt->execute();
        $result = $prepStmt->get_result();
        if ($result->num_rows === 1) {
            $row = $result->fetch_assoc();
            $sql = "DELETE FROM users WHERE id = ?";
            $prepStmt = $this->conn->prepare($sql);
            $prepStmt->bind_param('i', $userId);
            if ($prepStmt->execute()) {
                $response['success'] = true;
            }
        } else {
            throw new Exception(Errors::userError('unknownUser'), 422);
        }

        return $response;
    }

    public function usersStstistics(): array
    {
        $response = [];

        // get total users
        $sql = "SELECT id FROM users ";
        $prepStmt = $this->conn->prepare($sql);
        $prepStmt->execute();
        $result = $prepStmt->get_result();
        $response['totalUsers'] = $result->num_rows;
        $response['signUpCounts'] = $this->getSignupStatistics();
        $response['visitsCounts'] = $this->getVisitStatistics();

        return $response;
    }

    public function createInvestment(int $planId, float $amount, int $currencyId): array
    {
        $transactions = new Transactions();
        $response = $transactions->createInvestment($planId, $amount, $currencyId, $this->userInfo['id']);
        if ($response['error']) {
            throw  new Exception($response['error'], 422);
        } else {
            return ['success' => $response['success']];
        }
    }

    public function getInvestments(int $page, int $limit): array
    {
        $response = [];
        if ($this->userInfo !== null) {
            $transactions = new Transactions();
            $response = $transactions->getInvestments($page, $limit, $this->userInfo['id']);
        }
        return $response;
    }

    public function getInvestmentFullDetail(int $id): array
    {

        $transactions = new Transactions();
        $response = $transactions->investmentFullDetail($id);
        return $response;
    }

    public function requestWithdrawal(float $amount, int $currencyId): array
    {
        $response = [];
        if ($this->userInfo !== null) {
            $transactions = new Transactions();
            $response = $transactions->requestWithdrawal($amount, $currencyId, $this->userInfo['id']);
        }
        return $response;
    }

    public function confirmInvestmentPayment(int $id): array
    {
        $transactions = new Transactions();
        return $transactions->confirmInvestmentPayment($id);
    }

    public function getWithdrawals(int $page, int $limit): array
    {
        $response = [];
        if ($this->userInfo !== null) {
            $transactions = new Transactions();
            $response = $transactions->getWithdrawals($page, $limit, $this->userInfo['id']);
        }
        return $response;
    }
}
